//+-------------------------------------------------------------------+
//|	Reverse_Stochastic_Lines.mq4
//|	Based on Symphonie_Market_Emotion_Indicator Indicator
//| !!! this file is created with Tab size 4 and no space indentation
//+-------------------------------------------------------------------+
#property copyright "Comer"
#property link		""

#property indicator_chart_window

#property indicator_buffers	2

#property indicator_color1	DodgerBlue
#property indicator_style1	STYLE_SOLID
#property indicator_width1	2

#property indicator_color2	Red
#property indicator_style2	STYLE_SOLID
#property indicator_width2	2

extern int		period		= 7;
extern double	K			= 50.6; // percentage. normally 0-100
extern int		Right_Shift	= 2;

double ExtMapBuffer1[];
double ExtMapBuffer2[];

int BarNumber;
int BarIndex;


int init() {
	int line = 0;

	SetIndexStyle ( line, DRAW_LINE, STYLE_SOLID, 2 );	// Blue
	SetIndexBuffer( line, ExtMapBuffer1 );
	line++;
	
	SetIndexStyle ( line, DRAW_LINE, STYLE_SOLID, 2 );	// Red
	SetIndexBuffer( line, ExtMapBuffer2);
	line++;

	return(0);
}

int deinit() {
	return(0);
}

int start() {
	BarNumber = IndicatorCounted();	// [0 .. Bars]

	/*	'BarNumber' is the number of the LAST fake bar - the one that last time had index 0
		MT4 documentation states that the last bar is always considered NOT updated.
		So the 'BarNumber' may point to NEXT to LAST bar. However, there are different
		confusing scenarios around that, so it is safer to recalculate 1 more bar, potentially.
	*/

	// number of bars are faked at the end of the graph - always recalculate those
	// if not - the graph will be always filled as if period is 1.
	int fakeBars		= period - Right_Shift;
	int firstFakeNumber	= BarNumber - fakeBars - 1;

	// now add those bars to the list of bars to recalculate
	BarNumber = firstFakeNumber;
	
	for( BarIndex = Bars - BarNumber; BarIndex > 0; ) {
		BarIndex--;		// [Bars-1 .. 0]
		processBar();
		BarNumber++;
	}
   
	return(0);
}


void processBar() {
	int		frontLineDataIdx	= BarIndex + Right_Shift;
	
	if( frontLineDataIdx >= Bars )	// 'backLineDataIdx' being negative is fine - it will be processed correctly
		return;
	
	// 'backLineDataIdx' WILL become negative - these bars ARE FAKE and WILL BE recalculated (repainted)
	int		backLineDataIdx		= frontLineDataIdx - period;
	double	front				= getC( frontLineDataIdx );
	double	back				= getC( backLineDataIdx );

	draw( back, front, (backLineDataIdx <= 0) );
	
	return(0);
}

double getC( int index ) {
	int	p = period;
	
	if( index < 0 ) {
		p += index; // reduce period for bars that are right of "real" current. those bars ARE FAKE and will be recalculated on the next bar.
		index = 0;
		if(p <= 0)
			p = 1;
	}
	double	H = High[ iHighest(NULL, 0, MODE_HIGH, p, index ) ];
	double	L = Low [ iLowest (NULL, 0, MODE_LOW , p, index ) ];
	
	// stochastic is 'k = 100*(c-L)/(H-L)'
	// solving this equasion for 'c' gives 'c = L + (H-L)*k/100'
	// or 'H - (H-L)*K/100', where 'K = 100-k'
	double c = H - (H-L)*K/100;
	
	return (c);
}

void draw( double back, double front, bool isFake ) {
	ExtMapBuffer1[ BarIndex ] = back;
	ExtMapBuffer2[ BarIndex ] = front;
}

